@@ -3,16 +3,24 @@ Helpers for configuring locale settings.
 
 Name `localization` is chosen to avoid overlap with builtin `locale` module.
 """
+from __future__ import annotations
+
 from contextlib import contextmanager
 import locale
 import re
 import subprocess
+from typing import (
+    Callable,
+    Iterator,
+)
 
 from pandas._config.config import options
 
 
 @contextmanager
-def set_locale(new_locale, lc_var: int = locale.LC_ALL):
+def set_locale(
+    new_locale: str | tuple[str, str], lc_var: int = locale.LC_ALL
+) -> Iterator[str | tuple[str, str]]:
     """
     Context manager for temporarily setting a locale.
 
@@ -71,7 +79,7 @@ def can_set_locale(lc: str, lc_var: int = locale.LC_ALL) -> bool:
         return True
 
 
-def _valid_locales(locales, normalize):
+def _valid_locales(locales: list[str] | str, normalize: bool) -> list[str]:
     """
     Return a list of normalized locales that do not throw an ``Exception``
     when set.
@@ -88,20 +96,25 @@ def _valid_locales(locales, normalize):
     valid_locales : list
         A list of valid locales.
     """
-    if normalize:
-        normalizer = lambda x: locale.normalize(x.strip())
-    else:
-        normalizer = lambda x: x.strip()
-
-    return list(filter(can_set_locale, map(normalizer, locales)))
+    return [
+        loc
+        for loc in (
+            locale.normalize(loc.strip()) if normalize else loc.strip()
+            for loc in locales
+        )
+        if can_set_locale(loc)
+    ]
 
 
-def _default_locale_getter():
-    raw_locales = subprocess.check_output(["locale -a"], shell=True)
-    return raw_locales
+def _default_locale_getter() -> bytes:
+    return subprocess.check_output(["locale -a"], shell=True)
 
 
-def get_locales(prefix=None, normalize=True, locale_getter=_default_locale_getter):
+def get_locales(
+    prefix: str | None = None,
+    normalize: bool = True,
+    locale_getter: Callable[[], bytes] = _default_locale_getter,
+) -> list[str] | None:
     """
     Get all the locales that are available on the system.
 
@@ -141,9 +154,9 @@ def get_locales(prefix=None, normalize=True, locale_getter=_default_locale_gette
         # raw_locales is "\n" separated list of locales
         # it may contain non-decodable parts, so split
         # extract what we can and then rejoin.
-        raw_locales = raw_locales.split(b"\n")
+        split_raw_locales = raw_locales.split(b"\n")
         out_locales = []
-        for x in raw_locales:
+        for x in split_raw_locales:
             try:
                 out_locales.append(str(x, encoding=options.display.encoding))
             except UnicodeError:
