@@ -1,48 +1,66 @@
 """ manage PyTables query interface via Expressions """
+from __future__ import annotations
 
 import ast
 from functools import partial
-from typing import Any, Dict, Optional, Tuple
+from typing import Any
 
 import numpy as np
 
-from pandas._libs.tslibs import Timedelta, Timestamp
+from pandas._libs.tslibs import (
+    Timedelta,
+    Timestamp,
+)
+from pandas._typing import npt
 from pandas.compat.chainmap import DeepChainMap
 
 from pandas.core.dtypes.common import is_list_like
 
-import pandas as pd
 import pandas.core.common as com
-from pandas.core.computation import expr, ops, scope as _scope
-from pandas.core.computation.common import _ensure_decoded
+from pandas.core.computation import (
+    expr,
+    ops,
+    scope as _scope,
+)
+from pandas.core.computation.common import ensure_decoded
 from pandas.core.computation.expr import BaseExprVisitor
-from pandas.core.computation.ops import UndefinedVariableError, is_term
+from pandas.core.computation.ops import (
+    UndefinedVariableError,
+    is_term,
+)
 from pandas.core.construction import extract_array
+from pandas.core.indexes.base import Index
 
-from pandas.io.formats.printing import pprint_thing, pprint_thing_encoded
+from pandas.io.formats.printing import (
+    pprint_thing,
+    pprint_thing_encoded,
+)
 
 
 class PyTablesScope(_scope.Scope):
     __slots__ = ("queryables",)
 
-    queryables: Dict[str, Any]
+    queryables: dict[str, Any]
 
     def __init__(
         self,
         level: int,
         global_dict=None,
         local_dict=None,
-        queryables: Optional[Dict[str, Any]] = None,
+        queryables: dict[str, Any] | None = None,
     ):
         super().__init__(level + 1, global_dict=global_dict, local_dict=local_dict)
-        self.queryables = queryables or dict()
+        self.queryables = queryables or {}
 
 
 class Term(ops.Term):
     env: PyTablesScope
 
     def __new__(cls, name, env, side=None, encoding=None):
-        klass = Constant if not isinstance(name, str) else cls
+        if isinstance(name, str):
+            klass = cls
+        else:
+            klass = Constant
         return object.__new__(klass)
 
     def __init__(self, name, env: PyTablesScope, side=None, encoding=None):
@@ -63,7 +81,7 @@ class Term(ops.Term):
             return self.name
 
     # read-only property overwriting read/write property
-    @property  # type: ignore
+    @property  # type: ignore[misc]
     def value(self):
         return self._value
 
@@ -82,9 +100,10 @@ class BinOp(ops.BinOp):
     _max_selectors = 31
 
     op: str
-    queryables: Dict[str, Any]
+    queryables: dict[str, Any]
+    condition: str | None
 
-    def __init__(self, op: str, lhs, rhs, queryables: Dict[str, Any], encoding):
+    def __init__(self, op: str, lhs, rhs, queryables: dict[str, Any], encoding):
         super().__init__(op, lhs, rhs)
         self.queryables = queryables
         self.encoding = encoding
@@ -95,7 +114,7 @@ class BinOp(ops.BinOp):
 
     def prune(self, klass):
         def pr(left, right):
-            """ create and return a new specialized BinOp from myself """
+            """create and return a new specialized BinOp from myself"""
             if left is None:
                 return right
             elif right is None:
@@ -136,7 +155,7 @@ class BinOp(ops.BinOp):
         return res
 
     def conform(self, rhs):
-        """ inplace conform rhs """
+        """inplace conform rhs"""
         if not is_list_like(rhs):
             rhs = [rhs]
         if isinstance(rhs, np.ndarray):
@@ -145,7 +164,7 @@ class BinOp(ops.BinOp):
 
     @property
     def is_valid(self) -> bool:
-        """ return True if this is a valid field """
+        """return True if this is a valid field"""
         return self.lhs in self.queryables
 
     @property
@@ -158,25 +177,25 @@ class BinOp(ops.BinOp):
 
     @property
     def kind(self):
-        """ the kind of my field """
+        """the kind of my field"""
         return getattr(self.queryables.get(self.lhs), "kind", None)
 
     @property
     def meta(self):
-        """ the meta of my field """
+        """the meta of my field"""
         return getattr(self.queryables.get(self.lhs), "meta", None)
 
     @property
     def metadata(self):
-        """ the metadata of my field """
+        """the metadata of my field"""
         return getattr(self.queryables.get(self.lhs), "metadata", None)
 
     def generate(self, v) -> str:
-        """ create and return the op string for this TermValue """
+        """create and return the op string for this TermValue"""
         val = v.tostring(self.encoding)
         return f"({self.lhs} {self.op} {val})"
 
-    def convert_value(self, v) -> "TermValue":
+    def convert_value(self, v) -> TermValue:
         """
         convert the expression that is in the term to something that is
         accepted by pytables
@@ -184,17 +203,15 @@ class BinOp(ops.BinOp):
 
         def stringify(value):
             if self.encoding is not None:
-                encoder = partial(pprint_thing_encoded, encoding=self.encoding)
-            else:
-                encoder = pprint_thing
-            return encoder(value)
+                return pprint_thing_encoded(value, encoding=self.encoding)
+            return pprint_thing(value)
 
-        kind = _ensure_decoded(self.kind)
-        meta = _ensure_decoded(self.meta)
+        kind = ensure_decoded(self.kind)
+        meta = ensure_decoded(self.meta)
         if kind == "datetime64" or kind == "datetime":
             if isinstance(v, (int, float)):
                 v = stringify(v)
-            v = _ensure_decoded(v)
+            v = ensure_decoded(v)
             v = Timestamp(v)
             if v.tz is not None:
                 v = v.tz_convert("UTC")
@@ -207,12 +224,11 @@ class BinOp(ops.BinOp):
             return TermValue(int(v), v, kind)
         elif meta == "category":
             metadata = extract_array(self.metadata, extract_numpy=True)
-            result = metadata.searchsorted(v, side="left")
-
-            # result returns 0 if v is first element or if v is not in metadata
-            # check that metadata contains v
-            if not result and v not in metadata:
+            result: npt.NDArray[np.intp] | np.intp | int
+            if v not in metadata:
                 result = -1
+            else:
+                result = metadata.searchsorted(v, side="left")
             return TermValue(result, result, "integer")
         elif kind == "integer":
             v = int(float(v))
@@ -247,7 +263,7 @@ class BinOp(ops.BinOp):
 
 
 class FilterBinOp(BinOp):
-    filter: Optional[Tuple[Any, Any, pd.Index]] = None
+    filter: tuple[Any, Any, Index] | None = None
 
     def __repr__(self) -> str:
         if self.filter is None:
@@ -255,15 +271,17 @@ class FilterBinOp(BinOp):
         return pprint_thing(f"[Filter : [{self.filter[0]}] -> [{self.filter[1]}]")
 
     def invert(self):
-        """ invert the filter """
+        """invert the filter"""
         if self.filter is not None:
-            f = list(self.filter)
-            f[1] = self.generate_filter_op(invert=True)
-            self.filter = tuple(f)
+            self.filter = (
+                self.filter[0],
+                self.generate_filter_op(invert=True),
+                self.filter[2],
+            )
         return self
 
     def format(self):
-        """ return the actual filter format """
+        """return the actual filter format"""
         return [self.filter]
 
     def evaluate(self):
@@ -280,7 +298,7 @@ class FilterBinOp(BinOp):
             if self.op in ["==", "!="] and len(values) > self._max_selectors:
 
                 filter_op = self.generate_filter_op()
-                self.filter = (self.lhs, filter_op, pd.Index(values))
+                self.filter = (self.lhs, filter_op, Index(values))
 
                 return self
             return None
@@ -289,7 +307,7 @@ class FilterBinOp(BinOp):
         if self.op in ["==", "!="]:
 
             filter_op = self.generate_filter_op()
-            self.filter = (self.lhs, filter_op, pd.Index(values))
+            self.filter = (self.lhs, filter_op, Index(values))
 
         else:
             raise TypeError(
@@ -318,7 +336,7 @@ class ConditionBinOp(BinOp):
         return pprint_thing(f"[Condition : [{self.condition}]]")
 
     def invert(self):
-        """ invert the condition """
+        """invert the condition"""
         # if self.condition is not None:
         #    self.condition = "~(%s)" % self.condition
         # return self
@@ -327,7 +345,7 @@ class ConditionBinOp(BinOp):
         )
 
     def format(self):
-        """ return the actual ne format """
+        """return the actual ne format"""
         return self.condition
 
     def evaluate(self):
@@ -374,14 +392,14 @@ class UnaryOp(ops.UnaryOp):
         operand = self.operand
         operand = operand.prune(klass)
 
-        if operand is not None:
-            if issubclass(klass, ConditionBinOp):
-                if operand.condition is not None:
-                    return operand.invert()
-            elif issubclass(klass, FilterBinOp):
-                if operand.filter is not None:
-                    return operand.invert()
-
+        if operand is not None and (
+            issubclass(klass, ConditionBinOp)
+            and operand.condition is not None
+            or not issubclass(klass, ConditionBinOp)
+            and issubclass(klass, FilterBinOp)
+            and operand.filter is not None
+        ):
+            return operand.invert()
         return None
 
 
@@ -426,6 +444,10 @@ class PyTablesExprVisitor(BaseExprVisitor):
         except AttributeError:
             pass
 
+        if isinstance(slobj, Term):
+            # In py39 np.ndarray lookups with Term containing int raise
+            slobj = slobj.value
+
         try:
             return self.const_type(value[slobj], self.env)
         except TypeError as err:
@@ -520,13 +542,14 @@ class PyTablesExpr(expr.Expr):
     "major_axis>=20130101"
     """
 
-    _visitor: Optional[PyTablesExprVisitor]
+    _visitor: PyTablesExprVisitor | None
     env: PyTablesScope
+    expr: str
 
     def __init__(
         self,
         where,
-        queryables: Optional[Dict[str, Any]] = None,
+        queryables: dict[str, Any] | None = None,
         encoding=None,
         scope_level: int = 0,
     ):
@@ -540,13 +563,13 @@ class PyTablesExpr(expr.Expr):
         self._visitor = None
 
         # capture the environment if needed
-        local_dict: DeepChainMap[Any, Any] = DeepChainMap()
+        local_dict: DeepChainMap[Any, Any] | None = None
 
         if isinstance(where, PyTablesExpr):
             local_dict = where.env.scope
             _where = where.expr
 
-        elif isinstance(where, (list, tuple)):
+        elif is_list_like(where):
             where = list(where)
             for idx, w in enumerate(where):
                 if isinstance(w, PyTablesExpr):
@@ -554,8 +577,9 @@ class PyTablesExpr(expr.Expr):
                 else:
                     w = _validate_where(w)
                     where[idx] = w
-            _where = " & ".join((f"({w})" for w in com.flatten(where)))
+            _where = " & ".join([f"({w})" for w in com.flatten(where)])
         else:
+            # _validate_where ensures we otherwise have a string
             _where = where
 
         self.expr = _where
@@ -578,7 +602,7 @@ class PyTablesExpr(expr.Expr):
         return pprint_thing(self.expr)
 
     def evaluate(self):
-        """ create and return the numexpr condition and filter """
+        """create and return the numexpr condition and filter"""
         try:
             self.condition = self.terms.prune(ConditionBinOp)
         except AttributeError as err:
@@ -598,7 +622,7 @@ class PyTablesExpr(expr.Expr):
 
 
 class TermValue:
-    """ hold a term value the we use to construct a condition/filter """
+    """hold a term value the we use to construct a condition/filter"""
 
     def __init__(self, value, converted, kind: str):
         assert isinstance(kind, str), kind
@@ -607,7 +631,7 @@ class TermValue:
         self.kind = kind
 
     def tostring(self, encoding) -> str:
-        """ quote the string if not encoded else encode and return """
+        """quote the string if not encoded else encode and return"""
         if self.kind == "string":
             if encoding is not None:
                 return str(self.converted)
@@ -620,7 +644,7 @@ class TermValue:
 
 
 def maybe_expression(s) -> bool:
-    """ loose checking if s is a pytables-acceptable expression """
+    """loose checking if s is a pytables-acceptable expression"""
     if not isinstance(s, str):
         return False
     ops = PyTablesExprVisitor.binary_ops + PyTablesExprVisitor.unary_ops + ("=",)
