@@ -13,22 +13,53 @@ Partial documentation of the file format:
 Reference for binary data compression:
   http://collaboration.cmc.ec.gc.ca/science/rpn/biblio/ddj/Website/articles/CUJ/1992/9210/ross/ross.htm
 """
+from __future__ import annotations
+
 from collections import abc
-from datetime import datetime, timedelta
+from datetime import (
+    datetime,
+    timedelta,
+)
 import struct
+from typing import cast
 
 import numpy as np
 
-from pandas.errors import EmptyDataError, OutOfBoundsDatetime
+from pandas._typing import (
+    FilePath,
+    ReadBuffer,
+)
+from pandas.errors import (
+    EmptyDataError,
+    OutOfBoundsDatetime,
+)
 
 import pandas as pd
+from pandas import (
+    DataFrame,
+    isna,
+)
 
-from pandas.io.common import get_filepath_or_buffer
+from pandas.io.common import get_handle
 from pandas.io.sas._sas import Parser
 import pandas.io.sas.sas_constants as const
 from pandas.io.sas.sasreader import ReaderBase
 
 
+def _parse_datetime(sas_datetime: float, unit: str):
+    if isna(sas_datetime):
+        return pd.NaT
+
+    if unit == "s":
+        return datetime(1960, 1, 1) + timedelta(seconds=sas_datetime)
+
+    elif unit == "d":
+        return datetime(1960, 1, 1) + timedelta(days=sas_datetime)
+
+    else:
+        raise ValueError("unit must be 'd' or 's'")
+
+
 def _convert_datetimes(sas_datetimes: pd.Series, unit: str) -> pd.Series:
     """
     Convert to Timestamp if possible, otherwise to datetime.datetime.
@@ -50,24 +81,48 @@ def _convert_datetimes(sas_datetimes: pd.Series, unit: str) -> pd.Series:
     try:
         return pd.to_datetime(sas_datetimes, unit=unit, origin="1960-01-01")
     except OutOfBoundsDatetime:
-        if unit == "s":
-            return sas_datetimes.apply(
-                lambda sas_float: datetime(1960, 1, 1) + timedelta(seconds=sas_float)
-            )
-        elif unit == "d":
-            return sas_datetimes.apply(
-                lambda sas_float: datetime(1960, 1, 1) + timedelta(days=sas_float)
-            )
-        else:
-            raise ValueError("unit must be 'd' or 's'")
+        s_series = sas_datetimes.apply(_parse_datetime, unit=unit)
+        s_series = cast(pd.Series, s_series)
+        return s_series
 
 
-class _subheader_pointer:
-    pass
+class _SubheaderPointer:
+    offset: int
+    length: int
+    compression: int
+    ptype: int
+
+    def __init__(self, offset: int, length: int, compression: int, ptype: int):
+        self.offset = offset
+        self.length = length
+        self.compression = compression
+        self.ptype = ptype
 
 
-class _column:
-    pass
+class _Column:
+    col_id: int
+    name: str | bytes
+    label: str | bytes
+    format: str | bytes
+    ctype: bytes
+    length: int
+
+    def __init__(
+        self,
+        col_id: int,
+        # These can be bytes when convert_header_text is False
+        name: str | bytes,
+        label: str | bytes,
+        format: str | bytes,
+        ctype: bytes,
+        length: int,
+    ):
+        self.col_id = col_id
+        self.name = name
+        self.label = label
+        self.format = format
+        self.ctype = ctype
+        self.length = length
 
 
 # SAS7BDAT represents a SAS data file in SAS7BDAT format.
@@ -82,10 +137,10 @@ class SAS7BDATReader(ReaderBase, abc.Iterator):
         contents.
     index : column identifier, defaults to None
         Column to use as index.
-    convert_dates : boolean, defaults to True
+    convert_dates : bool, defaults to True
         Attempt to convert dates to Pandas datetime values.  Note that
         some rarely used SAS date formats may be unsupported.
-    blank_missing : boolean, defaults to True
+    blank_missing : bool, defaults to True
         Convert empty strings to missing values (SAS uses blanks to
         indicate missing character variables).
     chunksize : int, defaults to None
@@ -100,9 +155,12 @@ class SAS7BDATReader(ReaderBase, abc.Iterator):
         bytes.
     """
 
+    _int_length: int
+    _cached_page: bytes | None
+
     def __init__(
         self,
-        path_or_buf,
+        path_or_buf: FilePath | ReadBuffer[bytes],
         index=None,
         convert_dates=True,
         blank_missing=True,
@@ -121,58 +179,57 @@ class SAS7BDATReader(ReaderBase, abc.Iterator):
         self.convert_header_text = convert_header_text
 
         self.default_encoding = "latin-1"
-        self.compression = ""
-        self.column_names_strings = []
-        self.column_names = []
-        self.column_formats = []
-        self.columns = []
+        self.compression = b""
+        self.column_names_strings: list[str] = []
+        self.column_names: list[str] = []
+        self.column_formats: list[str] = []
+        self.columns: list[_Column] = []
 
-        self._current_page_data_subheader_pointers = []
+        self._current_page_data_subheader_pointers: list[_SubheaderPointer] = []
         self._cached_page = None
-        self._column_data_lengths = []
-        self._column_data_offsets = []
-        self._column_types = []
+        self._column_data_lengths: list[int] = []
+        self._column_data_offsets: list[int] = []
+        self._column_types: list[bytes] = []
 
         self._current_row_in_file_index = 0
         self._current_row_on_page_index = 0
         self._current_row_in_file_index = 0
 
-        self._path_or_buf, _, _, _ = get_filepath_or_buffer(path_or_buf)
-        if isinstance(self._path_or_buf, str):
-            self._path_or_buf = open(self._path_or_buf, "rb")
-            self.handle = self._path_or_buf
+        self.handles = get_handle(path_or_buf, "rb", is_text=False)
 
-        self._get_properties()
-        self._parse_metadata()
+        self._path_or_buf = self.handles.handle
 
-    def column_data_lengths(self):
+        try:
+            self._get_properties()
+            self._parse_metadata()
+        except Exception:
+            self.close()
+            raise
+
+    def column_data_lengths(self) -> np.ndarray:
         """Return a numpy int64 array of the column data lengths"""
         return np.asarray(self._column_data_lengths, dtype=np.int64)
 
-    def column_data_offsets(self):
+    def column_data_offsets(self) -> np.ndarray:
         """Return a numpy int64 array of the column offsets"""
         return np.asarray(self._column_data_offsets, dtype=np.int64)
 
-    def column_types(self):
+    def column_types(self) -> np.ndarray:
         """
         Returns a numpy character array of the column types:
            s (string) or d (double)
         """
         return np.asarray(self._column_types, dtype=np.dtype("S1"))
 
-    def close(self):
-        try:
-            self.handle.close()
-        except AttributeError:
-            pass
+    def close(self) -> None:
+        self.handles.close()
 
-    def _get_properties(self):
+    def _get_properties(self) -> None:
 
         # Check magic number
         self._path_or_buf.seek(0)
         self._cached_page = self._path_or_buf.read(288)
         if self._cached_page[0 : len(const.magic)] != const.magic:
-            self.close()
             raise ValueError("magic number mismatch (not a SAS file?)")
 
         # Get alignment information
@@ -247,8 +304,9 @@ class SAS7BDATReader(ReaderBase, abc.Iterator):
         # Read the rest of the header into cached_page.
         buf = self._path_or_buf.read(self.header_length - 288)
         self._cached_page += buf
-        if len(self._cached_page) != self.header_length:
-            self.close()
+        # error: Argument 1 to "len" has incompatible type "Optional[bytes]";
+        #  expected "Sized"
+        if len(self._cached_page) != self.header_length:  # type: ignore[arg-type]
             raise ValueError("The SAS7BDAT file appears to be truncated.")
 
         self._page_length = self._read_int(
@@ -302,11 +360,12 @@ class SAS7BDATReader(ReaderBase, abc.Iterator):
     def __next__(self):
         da = self.read(nrows=self.chunksize or 1)
         if da is None:
+            self.close()
             raise StopIteration
         return da
 
     # Read a single float of the given width (4 or 8).
-    def _read_float(self, offset, width):
+    def _read_float(self, offset: int, width: int):
         if width not in (4, 8):
             self.close()
             raise ValueError("invalid float width")
@@ -315,7 +374,7 @@ class SAS7BDATReader(ReaderBase, abc.Iterator):
         return struct.unpack(self.byte_order + fd, buf)[0]
 
     # Read a single signed integer of the given width (1, 2, 4 or 8).
-    def _read_int(self, offset, width):
+    def _read_int(self, offset: int, width: int) -> int:
         if width not in (1, 2, 4, 8):
             self.close()
             raise ValueError("invalid int width")
@@ -324,7 +383,7 @@ class SAS7BDATReader(ReaderBase, abc.Iterator):
         iv = struct.unpack(self.byte_order + it, buf)[0]
         return iv
 
-    def _read_bytes(self, offset, length):
+    def _read_bytes(self, offset: int, length: int):
         if self._cached_page is None:
             self._path_or_buf.seek(offset)
             buf = self._path_or_buf.read(length)
@@ -339,25 +398,24 @@ class SAS7BDATReader(ReaderBase, abc.Iterator):
                 raise ValueError("The cached page is too small.")
             return self._cached_page[offset : offset + length]
 
-    def _parse_metadata(self):
+    def _parse_metadata(self) -> None:
         done = False
         while not done:
             self._cached_page = self._path_or_buf.read(self._page_length)
             if len(self._cached_page) <= 0:
                 break
             if len(self._cached_page) != self._page_length:
-                self.close()
                 raise ValueError("Failed to read a meta data page from the SAS file.")
             done = self._process_page_meta()
 
-    def _process_page_meta(self):
+    def _process_page_meta(self) -> bool:
         self._read_page_header()
         pt = [const.page_meta_type, const.page_amd_type] + const.page_mix_types
         if self._current_page_type in pt:
             self._process_page_metadata()
         is_data_page = self._current_page_type & const.page_data_type
         is_mix_page = self._current_page_type in const.page_mix_types
-        return (
+        return bool(
             is_data_page
             or is_mix_page
             or self._current_page_data_subheader_pointers != []
@@ -374,7 +432,7 @@ class SAS7BDATReader(ReaderBase, abc.Iterator):
             tx, const.subheader_count_length
         )
 
-    def _process_page_metadata(self):
+    def _process_page_metadata(self) -> None:
         bit_offset = self._page_bit_offset
 
         for i in range(self._current_page_subheaders_count):
@@ -391,19 +449,22 @@ class SAS7BDATReader(ReaderBase, abc.Iterator):
             )
             self._process_subheader(subheader_index, pointer)
 
-    def _get_subheader_index(self, signature, compression, ptype):
+    def _get_subheader_index(self, signature: bytes, compression, ptype) -> int:
+        # TODO: return here could be made an enum
         index = const.subheader_signature_to_index.get(signature)
         if index is None:
             f1 = (compression == const.compressed_subheader_id) or (compression == 0)
             f2 = ptype == const.compressed_subheader_type
-            if (self.compression != "") and f1 and f2:
+            if (self.compression != b"") and f1 and f2:
                 index = const.SASIndex.data_subheader_index
             else:
                 self.close()
                 raise ValueError("Unknown subheader signature")
         return index
 
-    def _process_subheader_pointers(self, offset, subheader_pointer_index):
+    def _process_subheader_pointers(
+        self, offset: int, subheader_pointer_index: int
+    ) -> _SubheaderPointer:
 
         subheader_pointer_length = self._subheader_pointer_length
         total_offset = offset + subheader_pointer_length * subheader_pointer_index
@@ -419,19 +480,19 @@ class SAS7BDATReader(ReaderBase, abc.Iterator):
 
         subheader_type = self._read_int(total_offset, 1)
 
-        x = _subheader_pointer()
-        x.offset = subheader_offset
-        x.length = subheader_length
-        x.compression = subheader_compression
-        x.ptype = subheader_type
+        x = _SubheaderPointer(
+            subheader_offset, subheader_length, subheader_compression, subheader_type
+        )
 
         return x
 
-    def _read_subheader_signature(self, offset):
+    def _read_subheader_signature(self, offset: int) -> bytes:
         subheader_signature = self._read_bytes(offset, self._int_length)
         return subheader_signature
 
-    def _process_subheader(self, subheader_index, pointer):
+    def _process_subheader(
+        self, subheader_index: int, pointer: _SubheaderPointer
+    ) -> None:
         offset = pointer.offset
         length = pointer.length
 
@@ -459,7 +520,7 @@ class SAS7BDATReader(ReaderBase, abc.Iterator):
 
         processor(offset, length)
 
-    def _process_rowsize_subheader(self, offset, length):
+    def _process_rowsize_subheader(self, offset: int, length: int) -> None:
 
         int_len = self._int_length
         lcs_offset = offset
@@ -488,7 +549,7 @@ class SAS7BDATReader(ReaderBase, abc.Iterator):
         self._lcs = self._read_int(lcs_offset, 2)
         self._lcp = self._read_int(lcp_offset, 2)
 
-    def _process_columnsize_subheader(self, offset, length):
+    def _process_columnsize_subheader(self, offset: int, length: int) -> None:
         int_len = self._int_length
         offset += int_len
         self.column_count = self._read_int(offset, int_len)
@@ -499,10 +560,10 @@ class SAS7BDATReader(ReaderBase, abc.Iterator):
             )
 
     # Unknown purpose
-    def _process_subheader_counts(self, offset, length):
+    def _process_subheader_counts(self, offset: int, length: int) -> None:
         pass
 
-    def _process_columntext_subheader(self, offset, length):
+    def _process_columntext_subheader(self, offset: int, length: int) -> None:
 
         offset += self._int_length
         text_block_size = self._read_int(offset, const.text_block_size_length)
@@ -515,7 +576,7 @@ class SAS7BDATReader(ReaderBase, abc.Iterator):
         self.column_names_strings.append(cname)
 
         if len(self.column_names_strings) == 1:
-            compression_literal = ""
+            compression_literal = b""
             for cl in const.compression_literals:
                 if cl in cname_raw:
                     compression_literal = cl
@@ -528,7 +589,7 @@ class SAS7BDATReader(ReaderBase, abc.Iterator):
 
             buf = self._read_bytes(offset1, self._lcp)
             compression_literal = buf.rstrip(b"\x00")
-            if compression_literal == "":
+            if compression_literal == b"":
                 self._lcs = 0
                 offset1 = offset + 32
                 if self.U64:
@@ -554,7 +615,7 @@ class SAS7BDATReader(ReaderBase, abc.Iterator):
                         self.encoding or self.default_encoding
                     )
 
-    def _process_columnname_subheader(self, offset, length):
+    def _process_columnname_subheader(self, offset: int, length: int) -> None:
         int_len = self._int_length
         offset += int_len
         column_name_pointers_count = (length - 2 * int_len - 12) // 8
@@ -586,7 +647,7 @@ class SAS7BDATReader(ReaderBase, abc.Iterator):
             name_str = self.column_names_strings[idx]
             self.column_names.append(name_str[col_offset : col_offset + col_len])
 
-    def _process_columnattributes_subheader(self, offset, length):
+    def _process_columnattributes_subheader(self, offset: int, length: int) -> None:
         int_len = self._int_length
         column_attributes_vectors_count = (length - 2 * int_len - 12) // (int_len + 8)
         for i in range(column_attributes_vectors_count):
@@ -612,11 +673,11 @@ class SAS7BDATReader(ReaderBase, abc.Iterator):
             x = self._read_int(col_types, const.column_type_length)
             self._column_types.append(b"d" if x == 1 else b"s")
 
-    def _process_columnlist_subheader(self, offset, length):
+    def _process_columnlist_subheader(self, offset: int, length: int) -> None:
         # unknown purpose
         pass
 
-    def _process_format_subheader(self, offset, length):
+    def _process_format_subheader(self, offset: int, length: int) -> None:
         int_len = self._int_length
         text_subheader_format = (
             offset + const.column_format_text_subheader_index_offset + 3 * int_len
@@ -653,18 +714,19 @@ class SAS7BDATReader(ReaderBase, abc.Iterator):
         column_format = format_names[format_start : format_start + format_len]
         current_column_number = len(self.columns)
 
-        col = _column()
-        col.col_id = current_column_number
-        col.name = self.column_names[current_column_number]
-        col.label = column_label
-        col.format = column_format
-        col.ctype = self._column_types[current_column_number]
-        col.length = self._column_data_lengths[current_column_number]
+        col = _Column(
+            current_column_number,
+            self.column_names[current_column_number],
+            column_label,
+            column_format,
+            self._column_types[current_column_number],
+            self._column_data_lengths[current_column_number],
+        )
 
         self.column_formats.append(column_format)
         self.columns.append(col)
 
-    def read(self, nrows=None):
+    def read(self, nrows: int | None = None) -> DataFrame | None:
 
         if (nrows is None) and (self.chunksize is not None):
             nrows = self.chunksize
@@ -723,12 +785,12 @@ class SAS7BDATReader(ReaderBase, abc.Iterator):
 
         return False
 
-    def _chunk_to_dataframe(self):
+    def _chunk_to_dataframe(self) -> DataFrame:
 
         n = self._current_row_in_chunk_index
         m = self._current_row_in_file_index
         ix = range(m - n, m)
-        rslt = pd.DataFrame(index=ix)
+        rslt = {}
 
         js, jb = 0, 0
         for j in range(self.column_count):
@@ -736,8 +798,8 @@ class SAS7BDATReader(ReaderBase, abc.Iterator):
             name = self.column_names[j]
 
             if self._column_types[j] == b"d":
-                rslt[name] = self._byte_chunk[jb, :].view(dtype=self.byte_order + "d")
-                rslt[name] = np.asarray(rslt[name], dtype=np.float64)
+                col_arr = self._byte_chunk[jb, :].view(dtype=self.byte_order + "d")
+                rslt[name] = pd.Series(col_arr, dtype=np.float64, index=ix)
                 if self.convert_dates:
                     if self.column_formats[j] in const.sas_date_formats:
                         rslt[name] = _convert_datetimes(rslt[name], "d")
@@ -745,17 +807,18 @@ class SAS7BDATReader(ReaderBase, abc.Iterator):
                         rslt[name] = _convert_datetimes(rslt[name], "s")
                 jb += 1
             elif self._column_types[j] == b"s":
-                rslt[name] = self._string_chunk[js, :]
+                rslt[name] = pd.Series(self._string_chunk[js, :], index=ix)
                 if self.convert_text and (self.encoding is not None):
                     rslt[name] = rslt[name].str.decode(
                         self.encoding or self.default_encoding
                     )
                 if self.blank_missing:
                     ii = rslt[name].str.len() == 0
-                    rslt.loc[ii, name] = np.nan
+                    rslt[name][ii] = np.nan
                 js += 1
             else:
                 self.close()
-                raise ValueError(f"unknown column type {self._column_types[j]}")
+                raise ValueError(f"unknown column type {repr(self._column_types[j])}")
 
-        return rslt
+        df = DataFrame(rslt, columns=self.column_names, index=ix, copy=False)
+        return df
