@@ -73,7 +73,7 @@ class TestCategoricalIndexingWithFactor(TestCategorical):
         target = Categorical(["a", "b"], categories=["a", "b"])
         mask = np.array([True, False])
         msg = "Cannot set a Categorical with another, without identical categories"
-        with pytest.raises(ValueError, match=msg):
+        with pytest.raises(TypeError, match=msg):
             target[mask] = other[mask]
 
     @pytest.mark.parametrize(
@@ -89,7 +89,7 @@ class TestCategoricalIndexingWithFactor(TestCategorical):
         target = Categorical(["a", "b"], categories=["a", "b"], ordered=True)
         mask = np.array([True, False])
         msg = "Cannot set a Categorical with another, without identical categories"
-        with pytest.raises(ValueError, match=msg):
+        with pytest.raises(TypeError, match=msg):
             target[mask] = other[mask]
 
     def test_setitem_tuple(self):
@@ -105,15 +105,15 @@ class TestCategoricalIndexingWithFactor(TestCategorical):
         # GH#9469
         # properly coerce the input indexers
         np.random.seed(1)
-        c = Categorical(
+        cat = Categorical(
             np.random.randint(0, 5, size=150000).astype(np.int8)
         ).add_categories([-1000])
         indexer = np.array([100000]).astype(np.int64)
-        c[indexer] = -1000
+        cat[indexer] = -1000
 
         # we are asserting the code result here
         # which maps to the -1000 category
-        result = c.codes[np.array([100000]).astype(np.int64)]
+        result = cat.codes[np.array([100000]).astype(np.int64)]
         tm.assert_numpy_array_equal(result, np.array([5], dtype="int8"))
 
 
@@ -189,11 +189,11 @@ class TestCategoricalIndexing:
         tm.assert_index_equal(cat3.categories, exp_idx)
 
     def test_categories_assignments(self):
-        s = Categorical(["a", "b", "c", "a"])
+        cat = Categorical(["a", "b", "c", "a"])
         exp = np.array([1, 2, 3, 1], dtype=np.int64)
-        s.categories = [1, 2, 3]
-        tm.assert_numpy_array_equal(s.__array__(), exp)
-        tm.assert_index_equal(s.categories, Index([1, 2, 3]))
+        cat.categories = [1, 2, 3]
+        tm.assert_numpy_array_equal(cat.__array__(), exp)
+        tm.assert_index_equal(cat.categories, Index([1, 2, 3]))
 
     @pytest.mark.parametrize("new_categories", [[1, 2, 3, 4], [1, 2]])
     def test_categories_assignments_wrong_length_raises(self, new_categories):
@@ -260,7 +260,7 @@ class TestCategoricalIndexing:
     def test_where_new_category_raises(self):
         ser = Series(Categorical(["a", "b", "c"]))
         msg = "Cannot setitem on a Categorical with a new category"
-        with pytest.raises(ValueError, match=msg):
+        with pytest.raises(TypeError, match=msg):
             ser.where([True, False, True], "d")
 
     def test_where_ordered_differs_rasies(self):
@@ -270,27 +270,27 @@ class TestCategoricalIndexing:
         other = Categorical(
             ["b", "c", "a"], categories=["a", "c", "b", "d"], ordered=True
         )
-        with pytest.raises(ValueError, match="without identical categories"):
+        with pytest.raises(TypeError, match="without identical categories"):
             ser.where([True, False, True], other)
 
 
 class TestContains:
     def test_contains(self):
         # GH#21508
-        c = Categorical(list("aabbca"), categories=list("cab"))
+        cat = Categorical(list("aabbca"), categories=list("cab"))
 
-        assert "b" in c
-        assert "z" not in c
-        assert np.nan not in c
+        assert "b" in cat
+        assert "z" not in cat
+        assert np.nan not in cat
         with pytest.raises(TypeError, match="unhashable type: 'list'"):
-            assert [1] in c
+            assert [1] in cat
 
         # assert codes NOT in index
-        assert 0 not in c
-        assert 1 not in c
+        assert 0 not in cat
+        assert 1 not in cat
 
-        c = Categorical(list("aabbca") + [np.nan], categories=list("cab"))
-        assert np.nan in c
+        cat = Categorical(list("aabbca") + [np.nan], categories=list("cab"))
+        assert np.nan in cat
 
     @pytest.mark.parametrize(
         "item, expected",
@@ -325,27 +325,27 @@ class TestContains:
 
 @pytest.mark.parametrize("index", [True, False])
 def test_mask_with_boolean(index):
-    s = Series(range(3))
+    ser = Series(range(3))
     idx = Categorical([True, False, True])
     if index:
         idx = CategoricalIndex(idx)
 
     assert com.is_bool_indexer(idx)
-    result = s[idx]
-    expected = s[idx.astype("object")]
+    result = ser[idx]
+    expected = ser[idx.astype("object")]
     tm.assert_series_equal(result, expected)
 
 
 @pytest.mark.parametrize("index", [True, False])
 def test_mask_with_boolean_na_treated_as_false(index):
     # https://github.com/pandas-dev/pandas/issues/31503
-    s = Series(range(3))
+    ser = Series(range(3))
     idx = Categorical([True, False, None])
     if index:
         idx = CategoricalIndex(idx)
 
-    result = s[idx]
-    expected = s[idx.fillna(False)]
+    result = ser[idx]
+    expected = ser[idx.fillna(False)]
 
     tm.assert_series_equal(result, expected)
 
