@@ -9,18 +9,17 @@ import numpy as np
 import pytest
 
 from pandas.core.dtypes.cast import find_common_type
+from pandas.core.dtypes.common import is_dtype_equal
 
 from pandas import (
     CategoricalIndex,
     DatetimeIndex,
-    Float64Index,
     Index,
-    Int64Index,
     MultiIndex,
+    RangeIndex,
     Series,
     TimedeltaIndex,
     Timestamp,
-    UInt64Index,
 )
 import pandas._testing as tm
 from pandas.api.types import (
@@ -28,6 +27,11 @@ from pandas.api.types import (
     is_signed_integer_dtype,
     pandas_dtype,
 )
+from pandas.core.api import (
+    Float64Index,
+    Int64Index,
+    UInt64Index,
+)
 
 COMPATIBLE_INCONSISTENT_PAIRS = [
     (np.float64, np.int64),
@@ -43,12 +47,24 @@ def test_union_same_types(index):
     assert idx1.union(idx2).dtype == idx1.dtype
 
 
-def test_union_different_types(index_flat, index_flat2):
+def test_union_different_types(index_flat, index_flat2, request):
     # This test only considers combinations of indices
     # GH 23525
     idx1 = index_flat
     idx2 = index_flat2
 
+    if (
+        not idx1.is_unique
+        and idx1.dtype.kind == "i"
+        and is_dtype_equal(idx2.dtype, "boolean")
+    ) or (
+        not idx2.is_unique
+        and idx2.dtype.kind == "i"
+        and is_dtype_equal(idx1.dtype, "boolean")
+    ):
+        mark = pytest.mark.xfail(reason="GH#44000 True==1", raises=ValueError)
+        request.node.add_marker(mark)
+
     common_dtype = find_common_type([idx1.dtype, idx2.dtype])
 
     any_uint64 = idx1.dtype == np.uint64 or idx2.dtype == np.uint64
@@ -178,7 +194,7 @@ class TestSetOps:
             return
 
         # GH#10149
-        cases = [klass(second.values) for klass in [np.array, Series, list]]
+        cases = [second.to_numpy(), second.to_series(), second.to_list()]
         for case in cases:
             result = first.intersection(case)
             assert tm.equalContents(result, second)
@@ -192,6 +208,7 @@ class TestSetOps:
         first = index[3:]
         second = index[:5]
         everything = index
+
         union = first.union(second)
         assert tm.equalContents(union, everything)
 
@@ -201,15 +218,10 @@ class TestSetOps:
             return
 
         # GH#10149
-        cases = [klass(second.values) for klass in [np.array, Series, list]]
+        cases = [second.to_numpy(), second.to_series(), second.to_list()]
         for case in cases:
-            if not isinstance(index, CategoricalIndex):
-                result = first.union(case)
-                assert tm.equalContents(result, everything), (
-                    result,
-                    everything,
-                    type(case),
-                )
+            result = first.union(case)
+            assert tm.equalContents(result, everything)
 
         if isinstance(index, MultiIndex):
             msg = "other must be a MultiIndex or a list of tuples"
@@ -227,16 +239,10 @@ class TestSetOps:
         assert tm.equalContents(result, answer)
 
         # GH#10149
-        cases = [klass(second.values) for klass in [np.array, Series, list]]
+        cases = [second.to_numpy(), second.to_series(), second.to_list()]
         for case in cases:
-            if isinstance(index, (DatetimeIndex, TimedeltaIndex)):
-                assert type(result) == type(answer)
-                tm.assert_numpy_array_equal(
-                    result.sort_values().asi8, answer.sort_values().asi8
-                )
-            else:
-                result = first.difference(case, sort)
-                assert tm.equalContents(result, answer)
+            result = first.difference(case, sort)
+            assert tm.equalContents(result, answer)
 
         if isinstance(index, MultiIndex):
             msg = "other must be a MultiIndex or a list of tuples"
@@ -260,16 +266,9 @@ class TestSetOps:
         assert tm.equalContents(result, answer)
 
         # GH#10149
-        cases = [klass(second.values) for klass in [np.array, Series, list]]
+        cases = [second.to_numpy(), second.to_series(), second.to_list()]
         for case in cases:
             result = first.symmetric_difference(case)
-
-            if is_datetime64tz_dtype(first):
-                # second.values casts to tznaive
-                expected = first.union(case)
-                tm.assert_index_equal(result, expected)
-                continue
-
             assert tm.equalContents(result, answer)
 
         if isinstance(index, MultiIndex):
@@ -287,13 +286,11 @@ class TestSetOps:
             (None, None, None),
         ],
     )
-    def test_corner_union(self, index_flat, fname, sname, expected_name):
+    def test_corner_union(self, index_flat_unique, fname, sname, expected_name):
         # GH#9943, GH#9862
         # Test unions with various name combinations
         # Do not test MultiIndex or repeats
-        index = index_flat
-        if not index.is_unique:
-            pytest.skip("Not for MultiIndex or repeated indices")
+        index = index_flat_unique
 
         # Test copy.union(copy)
         first = index.copy().set_names(fname)
@@ -333,10 +330,8 @@ class TestSetOps:
             (None, None, None),
         ],
     )
-    def test_union_unequal(self, index_flat, fname, sname, expected_name):
-        index = index_flat
-        if not index.is_unique:
-            pytest.skip("Not for MultiIndex or repeated indices")
+    def test_union_unequal(self, index_flat_unique, fname, sname, expected_name):
+        index = index_flat_unique
 
         # test copy.union(subset) - need sort for unicode and string
         first = index.copy().set_names(fname)
@@ -355,12 +350,10 @@ class TestSetOps:
             (None, None, None),
         ],
     )
-    def test_corner_intersect(self, index_flat, fname, sname, expected_name):
+    def test_corner_intersect(self, index_flat_unique, fname, sname, expected_name):
         # GH#35847
         # Test intersections with various name combinations
-        index = index_flat
-        if not index.is_unique:
-            pytest.skip("Not for MultiIndex or repeated indices")
+        index = index_flat_unique
 
         # Test copy.intersection(copy)
         first = index.copy().set_names(fname)
@@ -400,10 +393,8 @@ class TestSetOps:
             (None, None, None),
         ],
     )
-    def test_intersect_unequal(self, index_flat, fname, sname, expected_name):
-        index = index_flat
-        if not index.is_unique:
-            pytest.skip("Not for MultiIndex or repeated indices")
+    def test_intersect_unequal(self, index_flat_unique, fname, sname, expected_name):
+        index = index_flat_unique
 
         # test copy.intersection(subset) - need sort for unicode and string
         first = index.copy().set_names(fname)
@@ -468,20 +459,20 @@ class TestSetOps:
 @pytest.mark.parametrize(
     "method", ["intersection", "union", "difference", "symmetric_difference"]
 )
-def test_setop_with_categorical(index, sort, method):
-    if isinstance(index, MultiIndex):
-        # tested separately in tests.indexes.multi.test_setops
-        return
+def test_setop_with_categorical(index_flat, sort, method):
+    # MultiIndex tested separately in tests.indexes.multi.test_setops
+    index = index_flat
 
     other = index.astype("category")
+    exact = "equiv" if isinstance(index, RangeIndex) else True
 
     result = getattr(index, method)(other, sort=sort)
     expected = getattr(index, method)(index, sort=sort)
-    tm.assert_index_equal(result, expected)
+    tm.assert_index_equal(result, expected, exact=exact)
 
     result = getattr(index, method)(other[:5], sort=sort)
     expected = getattr(index, method)(index[:5], sort=sort)
-    tm.assert_index_equal(result, expected)
+    tm.assert_index_equal(result, expected, exact=exact)
 
 
 def test_intersection_duplicates_all_indexes(index):
@@ -787,7 +778,7 @@ class TestSetOpsUnsorted:
     @pytest.mark.xfail(reason="Not implemented")
     @pytest.mark.parametrize("opname", ["difference", "symmetric_difference"])
     def test_difference_incomparable_true(self, opname):
-        # TODO: decide on True behaviour
+        # TODO(GH#25151): decide on True behaviour
         # # sort=True, raises
         a = Index([3, Timestamp("2000"), 1])
         b = Index([2, Timestamp("1999"), 1])
