@@ -6,7 +6,12 @@ import numpy as np
 import pytest
 
 import pandas as pd
-from pandas import DataFrame, Index, MultiIndex, option_context
+from pandas import (
+    DataFrame,
+    Index,
+    MultiIndex,
+    option_context,
+)
 import pandas._testing as tm
 
 import pandas.io.formats.format as fmt
@@ -137,7 +142,7 @@ def test_to_html_encoding(float_frame, tmp_path):
     # GH 28663
     path = tmp_path / "test.html"
     float_frame.to_html(path, encoding="gbk")
-    with open(str(path), "r", encoding="gbk") as f:
+    with open(str(path), encoding="gbk") as f:
         assert float_frame.to_html() == f.read()
 
 
@@ -152,8 +157,8 @@ def test_to_html_decimal(datapath):
 @pytest.mark.parametrize(
     "kwargs,string,expected",
     [
-        (dict(), "<type 'str'>", "escaped"),
-        (dict(escape=False), "<b>bold</b>", "escape_disabled"),
+        ({}, "<type 'str'>", "escaped"),
+        ({"escape": False}, "<b>bold</b>", "escape_disabled"),
     ],
 )
 def test_to_html_escaped(kwargs, string, expected, datapath):
@@ -247,6 +252,21 @@ def test_to_html_multiindex_odd_even_truncate(max_rows, expected, datapath):
             {"hod": lambda x: x.strftime("%H:%M")},
             "datetime64_hourformatter",
         ),
+        (
+            DataFrame(
+                {
+                    "i": pd.Series([1, 2], dtype="int64"),
+                    "f": pd.Series([1, 2], dtype="float64"),
+                    "I": pd.Series([1, 2], dtype="Int64"),
+                    "s": pd.Series([1, 2], dtype="string"),
+                    "b": pd.Series([True, False], dtype="boolean"),
+                    "c": pd.Series(["a", "b"], dtype=pd.CategoricalDtype(["a", "b"])),
+                    "o": pd.Series([1, "2"], dtype=object),
+                }
+            ),
+            [lambda x: "formatted"] * 7,
+            "various_dtypes_formatted",
+        ),
     ],
 )
 def test_to_html_formatters(df, formatters, expected, datapath):
@@ -432,43 +452,59 @@ def test_to_html_invalid_justify(justify):
         df.to_html(justify=justify)
 
 
-def test_to_html_index(datapath):
-    # TODO: split this test
-    index = ["foo", "bar", "baz"]
-    df = DataFrame(
-        {"A": [1, 2, 3], "B": [1.2, 3.4, 5.6], "C": ["one", "two", np.nan]},
-        columns=["A", "B", "C"],
-        index=index,
-    )
-    expected_with_index = expected_html(datapath, "index_1")
-    assert df.to_html() == expected_with_index
+class TestHTMLIndex:
+    @pytest.fixture
+    def df(self):
+        index = ["foo", "bar", "baz"]
+        df = DataFrame(
+            {"A": [1, 2, 3], "B": [1.2, 3.4, 5.6], "C": ["one", "two", np.nan]},
+            columns=["A", "B", "C"],
+            index=index,
+        )
+        return df
 
-    expected_without_index = expected_html(datapath, "index_2")
-    result = df.to_html(index=False)
-    for i in index:
-        assert i not in result
-    assert result == expected_without_index
-    df.index = Index(["foo", "bar", "baz"], name="idx")
-    expected_with_index = expected_html(datapath, "index_3")
-    assert df.to_html() == expected_with_index
-    assert df.to_html(index=False) == expected_without_index
+    @pytest.fixture
+    def expected_without_index(self, datapath):
+        return expected_html(datapath, "index_2")
 
-    tuples = [("foo", "car"), ("foo", "bike"), ("bar", "car")]
-    df.index = MultiIndex.from_tuples(tuples)
+    def test_to_html_flat_index_without_name(
+        self, datapath, df, expected_without_index
+    ):
+        expected_with_index = expected_html(datapath, "index_1")
+        assert df.to_html() == expected_with_index
 
-    expected_with_index = expected_html(datapath, "index_4")
-    assert df.to_html() == expected_with_index
+        result = df.to_html(index=False)
+        for i in df.index:
+            assert i not in result
+        assert result == expected_without_index
 
-    result = df.to_html(index=False)
-    for i in ["foo", "bar", "car", "bike"]:
-        assert i not in result
-    # must be the same result as normal index
-    assert result == expected_without_index
+    def test_to_html_flat_index_with_name(self, datapath, df, expected_without_index):
+        df.index = Index(["foo", "bar", "baz"], name="idx")
+        expected_with_index = expected_html(datapath, "index_3")
+        assert df.to_html() == expected_with_index
+        assert df.to_html(index=False) == expected_without_index
 
-    df.index = MultiIndex.from_tuples(tuples, names=["idx1", "idx2"])
-    expected_with_index = expected_html(datapath, "index_5")
-    assert df.to_html() == expected_with_index
-    assert df.to_html(index=False) == expected_without_index
+    def test_to_html_multiindex_without_names(
+        self, datapath, df, expected_without_index
+    ):
+        tuples = [("foo", "car"), ("foo", "bike"), ("bar", "car")]
+        df.index = MultiIndex.from_tuples(tuples)
+
+        expected_with_index = expected_html(datapath, "index_4")
+        assert df.to_html() == expected_with_index
+
+        result = df.to_html(index=False)
+        for i in ["foo", "bar", "car", "bike"]:
+            assert i not in result
+        # must be the same result as normal index
+        assert result == expected_without_index
+
+    def test_to_html_multiindex_with_names(self, datapath, df, expected_without_index):
+        tuples = [("foo", "car"), ("foo", "bike"), ("bar", "car")]
+        df.index = MultiIndex.from_tuples(tuples, names=["idx1", "idx2"])
+        expected_with_index = expected_html(datapath, "index_5")
+        assert df.to_html() == expected_with_index
+        assert df.to_html(index=False) == expected_without_index
 
 
 @pytest.mark.parametrize("classes", ["sortable draggable", ["sortable", "draggable"]])
@@ -743,7 +779,7 @@ def test_to_html_render_links(render_links, expected, datapath):
 def test_ignore_display_max_colwidth(method, expected, max_colwidth):
     # see gh-17004
     df = DataFrame([lorem_ipsum])
-    with pd.option_context("display.max_colwidth", max_colwidth):
+    with option_context("display.max_colwidth", max_colwidth):
         result = getattr(df, method)()
     expected = expected(max_colwidth)
     assert expected in result
@@ -762,7 +798,7 @@ def test_to_html_invalid_classes_type(classes):
 def test_to_html_round_column_headers():
     # GH 17280
     df = DataFrame([1], columns=[0.55555])
-    with pd.option_context("display.precision", 3):
+    with option_context("display.precision", 3):
         html = df.to_html(notebook=False)
         notebook = df.to_html(notebook=True)
     assert "0.55555" in html
@@ -787,13 +823,13 @@ def test_html_repr_min_rows_default(datapath):
     # gh-27991
 
     # default setting no truncation even if above min_rows
-    df = pd.DataFrame({"a": range(20)})
+    df = DataFrame({"a": range(20)})
     result = df._repr_html_()
     expected = expected_html(datapath, "html_repr_min_rows_default_no_truncation")
     assert result == expected
 
     # default of max_rows 60 triggers truncation if above
-    df = pd.DataFrame({"a": range(61)})
+    df = DataFrame({"a": range(61)})
     result = df._repr_html_()
     expected = expected_html(datapath, "html_repr_min_rows_default_truncated")
     assert result == expected
@@ -815,8 +851,40 @@ def test_html_repr_min_rows_default(datapath):
 def test_html_repr_min_rows(datapath, max_rows, min_rows, expected):
     # gh-27991
 
-    df = pd.DataFrame({"a": range(61)})
+    df = DataFrame({"a": range(61)})
     expected = expected_html(datapath, expected)
     with option_context("display.max_rows", max_rows, "display.min_rows", min_rows):
         result = df._repr_html_()
     assert result == expected
+
+
+def test_to_html_multilevel(multiindex_year_month_day_dataframe_random_data):
+    ymd = multiindex_year_month_day_dataframe_random_data
+
+    ymd.columns.name = "foo"
+    ymd.to_html()
+    ymd.T.to_html()
+
+
+@pytest.mark.parametrize("na_rep", ["NaN", "Ted"])
+def test_to_html_na_rep_and_float_format(na_rep, datapath):
+    # https://github.com/pandas-dev/pandas/issues/13828
+    df = DataFrame(
+        [
+            ["A", 1.2225],
+            ["A", None],
+        ],
+        columns=["Group", "Data"],
+    )
+    result = df.to_html(na_rep=na_rep, float_format="{:.2f}".format)
+    expected = expected_html(datapath, "gh13828_expected_output")
+    expected = expected.format(na_rep=na_rep)
+    assert result == expected
+
+
+def test_to_html_float_format_object_col(datapath):
+    # GH#40024
+    df = DataFrame(data={"x": [1000.0, "test"]})
+    result = df.to_html(float_format=lambda x: f"{x:,.0f}")
+    expected = expected_html(datapath, "gh40024_expected_output")
+    assert result == expected
