@@ -5,7 +5,6 @@ import json
 import locale
 import math
 import re
-import sys
 import time
 
 import dateutil
@@ -14,10 +13,23 @@ import pytest
 import pytz
 
 import pandas._libs.json as ujson
-from pandas._libs.tslib import Timestamp
-import pandas.compat as compat
+from pandas.compat import (
+    IS64,
+    PY310,
+    is_platform_windows,
+)
 
-from pandas import DataFrame, DatetimeIndex, Index, NaT, Series, Timedelta, date_range
+from pandas import (
+    DataFrame,
+    DatetimeIndex,
+    Index,
+    NaT,
+    PeriodIndex,
+    Series,
+    Timedelta,
+    Timestamp,
+    date_range,
+)
 import pandas._testing as tm
 
 
@@ -53,7 +65,7 @@ def get_int32_compat_dtype(numpy, orient):
     # See GH#32527
     dtype = np.int64
     if not ((numpy is None or orient == "index") or (numpy is True and orient is None)):
-        if compat.is_platform_windows():
+        if is_platform_windows():
             dtype = np.int32
         else:
             dtype = np.intp
@@ -62,9 +74,7 @@ def get_int32_compat_dtype(numpy, orient):
 
 
 class TestUltraJSONTests:
-    @pytest.mark.skipif(
-        compat.is_platform_32bit(), reason="not compliant on 32-bit, xref #15865"
-    )
+    @pytest.mark.skipif(not IS64, reason="not compliant on 32-bit, xref #15865")
     def test_encode_decimal(self):
         sut = decimal.Decimal("1337.1337")
         encoded = ujson.encode(sut, double_precision=15)
@@ -239,12 +249,29 @@ class TestUltraJSONTests:
             assert rounded_input == json.loads(output)
             assert rounded_input == ujson.decode(output)
 
-    @pytest.mark.parametrize("invalid_val", [20, -1, "9", None])
+    @pytest.mark.parametrize(
+        "invalid_val",
+        [
+            20,
+            -1,
+            pytest.param(
+                "9",
+                marks=pytest.mark.xfail(PY310, reason="Failing on Python 3.10 GH41940"),
+            ),
+            pytest.param(
+                None,
+                marks=pytest.mark.xfail(PY310, reason="Failing on Python 3.10 GH41940"),
+            ),
+        ],
+    )
     def test_invalid_double_precision(self, invalid_val):
         double_input = 30.12345678901234567890
         expected_exception = ValueError if isinstance(invalid_val, int) else TypeError
-
-        with pytest.raises(expected_exception):
+        msg = (
+            r"Invalid value '.*' for option 'double_precision', max is '15'|"
+            r"an integer is required \(got type "
+        )
+        with pytest.raises(expected_exception, match=msg):
             ujson.encode(double_input, double_precision=invalid_val)
 
     def test_encode_string_conversion2(self):
@@ -402,13 +429,13 @@ class TestUltraJSONTests:
         stamp = Timestamp(val)
 
         roundtrip = ujson.decode(ujson.encode(val, date_unit="s"))
-        assert roundtrip == stamp.value // 10 ** 9
+        assert roundtrip == stamp.value // 10**9
 
         roundtrip = ujson.decode(ujson.encode(val, date_unit="ms"))
-        assert roundtrip == stamp.value // 10 ** 6
+        assert roundtrip == stamp.value // 10**6
 
         roundtrip = ujson.decode(ujson.encode(val, date_unit="us"))
-        assert roundtrip == stamp.value // 10 ** 3
+        assert roundtrip == stamp.value // 10**3
 
         roundtrip = ujson.decode(ujson.encode(val, date_unit="ns"))
         assert roundtrip == stamp.value
@@ -449,13 +476,13 @@ class TestUltraJSONTests:
         decoded_input.member = O2()
         decoded_input.member.member = decoded_input
 
-        with pytest.raises(OverflowError):
+        with pytest.raises(OverflowError, match="Maximum recursion level reached"):
             ujson.encode(decoded_input)
 
     def test_decode_jibberish(self):
         jibberish = "fdsa sda v9sa fdsa"
-
-        with pytest.raises(ValueError):
+        msg = "Unexpected character found when decoding 'false'"
+        with pytest.raises(ValueError, match=msg):
             ujson.decode(jibberish)
 
     @pytest.mark.parametrize(
@@ -468,12 +495,13 @@ class TestUltraJSONTests:
         ],
     )
     def test_decode_broken_json(self, broken_json):
-        with pytest.raises(ValueError):
+        msg = "Expected object or value"
+        with pytest.raises(ValueError, match=msg):
             ujson.decode(broken_json)
 
     @pytest.mark.parametrize("too_big_char", ["[", "{"])
     def test_decode_depth_too_big(self, too_big_char):
-        with pytest.raises(ValueError):
+        with pytest.raises(ValueError, match="Reached object decoding depth limit"):
             ujson.decode(too_big_char * (1024 * 1024))
 
     @pytest.mark.parametrize(
@@ -487,13 +515,27 @@ class TestUltraJSONTests:
         ],
     )
     def test_decode_bad_string(self, bad_string):
-        with pytest.raises(ValueError):
+        msg = (
+            "Unexpected character found when decoding|"
+            "Unmatched ''\"' when when decoding 'string'"
+        )
+        with pytest.raises(ValueError, match=msg):
             ujson.decode(bad_string)
 
-    @pytest.mark.parametrize("broken_json", ['{{1337:""}}', '{{"key":"}', "[[[true"])
-    def test_decode_broken_json_leak(self, broken_json):
+    @pytest.mark.parametrize(
+        "broken_json, err_msg",
+        [
+            (
+                '{{1337:""}}',
+                "Key name of object must be 'string' when decoding 'object'",
+            ),
+            ('{{"key":"}', "Unmatched ''\"' when when decoding 'string'"),
+            ("[[[true", "Unexpected character found when decoding array value (2)"),
+        ],
+    )
+    def test_decode_broken_json_leak(self, broken_json, err_msg):
         for _ in range(1000):
-            with pytest.raises(ValueError):
+            with pytest.raises(ValueError, match=re.escape(err_msg)):
                 ujson.decode(broken_json)
 
     @pytest.mark.parametrize(
@@ -505,7 +547,12 @@ class TestUltraJSONTests:
         ],
     )
     def test_decode_invalid_dict(self, invalid_dict):
-        with pytest.raises(ValueError):
+        msg = (
+            "Key name of object must be 'string' when decoding 'object'|"
+            "No ':' found when decoding object value|"
+            "Expected object or value"
+        )
+        with pytest.raises(ValueError, match=msg):
             ujson.decode(invalid_dict)
 
     @pytest.mark.parametrize(
@@ -552,24 +599,23 @@ class TestUltraJSONTests:
             np.array(long_input), ujson.decode(output, numpy=True, dtype=np.int64)
         )
 
-    def test_encode_long_conversion(self):
-        long_input = 9223372036854775807
+    @pytest.mark.parametrize("long_input", [9223372036854775807, 18446744073709551615])
+    def test_encode_long_conversion(self, long_input):
         output = ujson.encode(long_input)
 
         assert long_input == json.loads(output)
         assert output == json.dumps(long_input)
         assert long_input == ujson.decode(output)
 
-    @pytest.mark.parametrize("bigNum", [sys.maxsize + 1, -(sys.maxsize + 2)])
-    @pytest.mark.xfail(not compat.IS64, reason="GH-35288")
+    @pytest.mark.parametrize("bigNum", [2**64, -(2**63) - 1])
     def test_dumps_ints_larger_than_maxsize(self, bigNum):
-        # GH34395
-        bigNum = sys.maxsize + 1
         encoding = ujson.encode(bigNum)
         assert str(bigNum) == encoding
 
-        # GH20599
-        with pytest.raises(ValueError):
+        with pytest.raises(
+            ValueError,
+            match="Value is too big|Value is too small",
+        ):
             assert ujson.loads(encoding) == bigNum
 
     @pytest.mark.parametrize(
@@ -583,7 +629,7 @@ class TestUltraJSONTests:
         with pytest.raises(TypeError, match=msg):
             ujson.loads(None)
 
-    @pytest.mark.parametrize("val", [3590016419, 2 ** 31, 2 ** 32, (2 ** 32) - 1])
+    @pytest.mark.parametrize("val", [3590016419, 2**31, 2**32, (2**32) - 1])
     def test_decode_number_with_32bit_sign_bit(self, val):
         # Test that numbers that fit within 32 bits but would have the
         # sign bit set (2**31 <= x < 2**32) are decoded properly.
@@ -593,14 +639,14 @@ class TestUltraJSONTests:
     def test_encode_big_escape(self):
         # Make sure no Exception is raised.
         for _ in range(10):
-            base = "\u00e5".encode("utf-8")
+            base = "\u00e5".encode()
             escape_input = base * 1024 * 1024 * 2
             ujson.encode(escape_input)
 
     def test_decode_big_escape(self):
         # Make sure no Exception is raised.
         for _ in range(10):
-            base = "\u00e5".encode("utf-8")
+            base = "\u00e5".encode()
             quote = b'"'
 
             escape_input = quote + (base * 1024 * 1024 * 2) + quote
@@ -673,6 +719,21 @@ class TestUltraJSONTests:
             ujson.encode(obj_list, default_handler=str)
         )
 
+    def test_encode_object(self):
+        class _TestObject:
+            def __init__(self, a, b, _c, d):
+                self.a = a
+                self.b = b
+                self._c = _c
+                self.d = d
+
+            def e(self):
+                return 5
+
+        # JSON keys should be all non-callable non-underscore attributes, see GH-42768
+        test_object = _TestObject(a=1, b=2, _c=3, d=4)
+        assert ujson.decode(ujson.encode(test_object)) == {"a": 1, "b": 2, "d": 4}
+
 
 class TestNumpyJSONTests:
     @pytest.mark.parametrize("bool_input", [True, False])
@@ -687,55 +748,55 @@ class TestNumpyJSONTests:
         output = np.array(ujson.decode(ujson.encode(bool_array)), dtype=bool)
         tm.assert_numpy_array_equal(bool_array, output)
 
-    def test_int(self, any_int_dtype):
-        klass = np.dtype(any_int_dtype).type
+    def test_int(self, any_int_numpy_dtype):
+        klass = np.dtype(any_int_numpy_dtype).type
         num = klass(1)
 
         assert klass(ujson.decode(ujson.encode(num))) == num
 
-    def test_int_array(self, any_int_dtype):
+    def test_int_array(self, any_int_numpy_dtype):
         arr = np.arange(100, dtype=int)
-        arr_input = arr.astype(any_int_dtype)
+        arr_input = arr.astype(any_int_numpy_dtype)
 
         arr_output = np.array(
-            ujson.decode(ujson.encode(arr_input)), dtype=any_int_dtype
+            ujson.decode(ujson.encode(arr_input)), dtype=any_int_numpy_dtype
         )
         tm.assert_numpy_array_equal(arr_input, arr_output)
 
-    def test_int_max(self, any_int_dtype):
-        if any_int_dtype in ("int64", "uint64") and compat.is_platform_32bit():
+    def test_int_max(self, any_int_numpy_dtype):
+        if any_int_numpy_dtype in ("int64", "uint64") and not IS64:
             pytest.skip("Cannot test 64-bit integer on 32-bit platform")
 
-        klass = np.dtype(any_int_dtype).type
+        klass = np.dtype(any_int_numpy_dtype).type
 
         # uint64 max will always overflow,
         # as it's encoded to signed.
-        if any_int_dtype == "uint64":
+        if any_int_numpy_dtype == "uint64":
             num = np.iinfo("int64").max
         else:
-            num = np.iinfo(any_int_dtype).max
+            num = np.iinfo(any_int_numpy_dtype).max
 
         assert klass(ujson.decode(ujson.encode(num))) == num
 
-    def test_float(self, float_dtype):
-        klass = np.dtype(float_dtype).type
+    def test_float(self, float_numpy_dtype):
+        klass = np.dtype(float_numpy_dtype).type
         num = klass(256.2013)
 
         assert klass(ujson.decode(ujson.encode(num))) == num
 
-    def test_float_array(self, float_dtype):
+    def test_float_array(self, float_numpy_dtype):
         arr = np.arange(12.5, 185.72, 1.7322, dtype=float)
-        float_input = arr.astype(float_dtype)
+        float_input = arr.astype(float_numpy_dtype)
 
         float_output = np.array(
             ujson.decode(ujson.encode(float_input, double_precision=15)),
-            dtype=float_dtype,
+            dtype=float_numpy_dtype,
         )
         tm.assert_almost_equal(float_input, float_output)
 
-    def test_float_max(self, float_dtype):
-        klass = np.dtype(float_dtype).type
-        num = klass(np.finfo(float_dtype).max / 10)
+    def test_float_max(self, float_numpy_dtype):
+        klass = np.dtype(float_numpy_dtype).type
+        num = klass(np.finfo(float_numpy_dtype).max / 10)
 
         tm.assert_almost_equal(
             klass(ujson.decode(ujson.encode(num, double_precision=15))), num
@@ -759,10 +820,10 @@ class TestNumpyJSONTests:
     def test_array_list(self):
         arr_list = [
             "a",
-            list(),
-            dict(),
-            dict(),
-            list(),
+            [],
+            {},
+            {},
+            [],
             42,
             97.8,
             ["a", "b"],
@@ -791,21 +852,70 @@ class TestNumpyJSONTests:
             ujson.encode(np.array(1))
 
     @pytest.mark.parametrize(
-        "bad_input,exc_type,kwargs",
+        "bad_input,exc_type,err_msg,kwargs",
         [
-            ([{}, []], ValueError, {}),
-            ([42, None], TypeError, {}),
-            ([["a"], 42], ValueError, {}),
-            ([42, {}, "a"], TypeError, {}),
-            ([42, ["a"], 42], ValueError, {}),
-            (["a", "b", [], "c"], ValueError, {}),
-            ([{"a": "b"}], ValueError, dict(labelled=True)),
-            ({"a": {"b": {"c": 42}}}, ValueError, dict(labelled=True)),
-            ([{"a": 42, "b": 23}, {"c": 17}], ValueError, dict(labelled=True)),
+            (
+                [{}, []],
+                ValueError,
+                r"nesting not supported for object or variable length dtypes",
+                {},
+            ),
+            (
+                [42, None],
+                TypeError,
+                r"int\(\) argument must be a string, a bytes-like object or a( real)? "
+                r"number, not 'NoneType'",
+                {},
+            ),
+            (
+                [["a"], 42],
+                ValueError,
+                r"Cannot decode multidimensional arrays with variable length elements "
+                r"to numpy",
+                {},
+            ),
+            (
+                [42, {}, "a"],
+                TypeError,
+                r"int\(\) argument must be a string, a bytes-like object or a( real)? "
+                r"number, not 'dict'",
+                {},
+            ),
+            (
+                [42, ["a"], 42],
+                ValueError,
+                r"invalid literal for int\(\) with base 10: 'a'",
+                {},
+            ),
+            (
+                ["a", "b", [], "c"],
+                ValueError,
+                r"nesting not supported for object or variable length dtypes",
+                {},
+            ),
+            (
+                [{"a": "b"}],
+                ValueError,
+                r"Cannot decode multidimensional arrays with variable length elements "
+                r"to numpy",
+                {"labelled": True},
+            ),
+            (
+                {"a": {"b": {"c": 42}}},
+                ValueError,
+                r"labels only supported up to 2 dimensions",
+                {"labelled": True},
+            ),
+            (
+                [{"a": 42, "b": 23}, {"c": 17}],
+                ValueError,
+                r"cannot reshape array of size 3 into shape \(2,1\)",
+                {"labelled": True},
+            ),
         ],
     )
-    def test_array_numpy_except(self, bad_input, exc_type, kwargs):
-        with pytest.raises(exc_type):
+    def test_array_numpy_except(self, bad_input, exc_type, err_msg, kwargs):
+        with pytest.raises(exc_type, match=err_msg):
             ujson.decode(ujson.dumps(bad_input), numpy=True, **kwargs)
 
     def test_array_numpy_labelled(self):
@@ -854,8 +964,8 @@ class TestPandasJSONTests:
             columns=["x", "y", "z"],
             dtype=dtype,
         )
-        encode_kwargs = {} if orient is None else dict(orient=orient)
-        decode_kwargs = {} if numpy is None else dict(numpy=numpy)
+        encode_kwargs = {} if orient is None else {"orient": orient}
+        decode_kwargs = {} if numpy is None else {"numpy": numpy}
         assert (df.dtypes == dtype).all()
 
         output = ujson.decode(ujson.encode(df, **encode_kwargs), **decode_kwargs)
@@ -886,7 +996,7 @@ class TestPandasJSONTests:
         )
 
         nested = {"df1": df, "df2": df.copy()}
-        kwargs = {} if orient is None else dict(orient=orient)
+        kwargs = {} if orient is None else {"orient": orient}
 
         exp = {
             "df1": ujson.decode(ujson.encode(df, **kwargs)),
@@ -904,7 +1014,7 @@ class TestPandasJSONTests:
             columns=["x", "y", "z"],
             dtype=int,
         )
-        kwargs = {} if orient is None else dict(orient=orient)
+        kwargs = {} if orient is None else {"orient": orient}
 
         output = DataFrame(
             *ujson.decode(ujson.encode(df, **kwargs), numpy=True, labelled=True)
@@ -927,8 +1037,8 @@ class TestPandasJSONTests:
         ).sort_values()
         assert s.dtype == dtype
 
-        encode_kwargs = {} if orient is None else dict(orient=orient)
-        decode_kwargs = {} if numpy is None else dict(numpy=numpy)
+        encode_kwargs = {} if orient is None else {"orient": orient}
+        decode_kwargs = {} if numpy is None else {"numpy": numpy}
 
         output = ujson.decode(ujson.encode(s, **encode_kwargs), **decode_kwargs)
         assert s.dtype == dtype
@@ -955,7 +1065,7 @@ class TestPandasJSONTests:
             [10, 20, 30, 40, 50, 60], name="series", index=[6, 7, 8, 9, 10, 15]
         ).sort_values()
         nested = {"s1": s, "s2": s.copy()}
-        kwargs = {} if orient is None else dict(orient=orient)
+        kwargs = {} if orient is None else {"orient": orient}
 
         exp = {
             "s1": ujson.decode(ujson.encode(s, **kwargs)),
@@ -1012,7 +1122,7 @@ class TestPandasJSONTests:
     def test_datetime_index(self):
         date_unit = "ns"
 
-        # freq doesnt round-trip
+        # freq doesn't round-trip
         rng = DatetimeIndex(list(date_range("1/1/2000", periods=20)), freq=None)
         encoded = ujson.encode(rng, date_unit=date_unit)
 
@@ -1036,7 +1146,11 @@ class TestPandasJSONTests:
         ],
     )
     def test_decode_invalid_array(self, invalid_arr):
-        with pytest.raises(ValueError):
+        msg = (
+            "Expected object or value|Trailing data|"
+            "Unexpected character found when decoding array value"
+        )
+        with pytest.raises(ValueError, match=msg):
             ujson.decode(invalid_arr)
 
     @pytest.mark.parametrize("arr", [[], [31337]])
@@ -1047,23 +1161,28 @@ class TestPandasJSONTests:
     def test_decode_extreme_numbers(self, extreme_num):
         assert extreme_num == ujson.decode(str(extreme_num))
 
-    @pytest.mark.parametrize(
-        "too_extreme_num", ["9223372036854775808", "-90223372036854775809"]
-    )
+    @pytest.mark.parametrize("too_extreme_num", [f"{2**64}", f"{-2**63-1}"])
     def test_decode_too_extreme_numbers(self, too_extreme_num):
-        with pytest.raises(ValueError):
+        with pytest.raises(
+            ValueError,
+            match="Value is too big|Value is too small",
+        ):
             ujson.decode(too_extreme_num)
 
     def test_decode_with_trailing_whitespaces(self):
         assert {} == ujson.decode("{}\n\t ")
 
     def test_decode_with_trailing_non_whitespaces(self):
-        with pytest.raises(ValueError):
+        with pytest.raises(ValueError, match="Trailing data"):
             ujson.decode("{}\n\t a")
 
-    def test_decode_array_with_big_int(self):
-        with pytest.raises(ValueError):
-            ujson.loads("[18446098363113800555]")
+    @pytest.mark.parametrize("value", [f"{2**64}", f"{-2**63-1}"])
+    def test_decode_array_with_big_int(self, value):
+        with pytest.raises(
+            ValueError,
+            match="Value is too big|Value is too small",
+        ):
+            ujson.loads(value)
 
     @pytest.mark.parametrize(
         "float_number",
@@ -1124,3 +1243,9 @@ class TestPandasJSONTests:
         expected = f'"{td.isoformat()}"'
 
         assert result == expected
+
+    def test_encode_periodindex(self):
+        # GH 46683
+        p = PeriodIndex(["2022-04-06", "2022-04-07"], freq="D")
+        df = DataFrame(index=p)
+        assert df.to_json() == "{}"
