@@ -154,8 +154,8 @@ class TestSeriesPlots(TestPlotBase):
         abs(self.ts).plot(ax=ax1, kind="area")
         abs(self.ts).plot(ax=ax2, kind="area")
 
-        assert ax1._shared_y_axes.joined(ax1, ax2)
-        assert ax2._shared_y_axes.joined(ax1, ax2)
+        assert self.get_y_axis(ax1).joined(ax1, ax2)
+        assert self.get_y_axis(ax2).joined(ax1, ax2)
 
     def test_label(self):
         s = Series([1, 2])
@@ -720,14 +720,14 @@ class TestSeriesPlots(TestPlotBase):
 
         _check_plot_works(s.plot)
 
-    @pytest.mark.xfail
+    @pytest.mark.xfail(reason="GH#24426")
     def test_plot_accessor_updates_on_inplace(self):
-        s = Series([1, 2, 3, 4])
+        ser = Series([1, 2, 3, 4])
         _, ax = self.plt.subplots()
-        ax = s.plot(ax=ax)
+        ax = ser.plot(ax=ax)
         before = ax.xaxis.get_ticklocs()
 
-        s.drop([0, 1], inplace=True)
+        ser.drop([0, 1], inplace=True)
         _, ax = self.plt.subplots()
         after = ax.xaxis.get_ticklocs()
         tm.assert_numpy_array_equal(before, after)
